// EXAMPLE 3
// simple 2d transformation and mouse drag event
#include <iostream>
#include <gl/glut.h>
using namespace std;

// rectangle position and size
float center[2] = {0.0, 0.0};
float center0[2];
float ratio = 0.5;
float ratio0;
// left/right mouse button flag
int lmbDown = 0;
int rmbDown = 0;

int winWidth = 600, winHeight = 600;
float oldPos[2], newPos[2];

void init( void )
{
	// set background to white
	glClearColor(1.0, 1.0, 1.0, 0.0);

	glMatrixMode (GL_PROJECTION);
	gluOrtho2D(-1.0, 1.0, -1.0, 1.0);
	glMatrixMode (GL_MODELVIEW);
}

// windows position -> world coordinate conversion
void win2world(float* worldpos, int x, int y)
{
	worldpos[0] = (float)x / winWidth * 2 - 1.0;
	worldpos[1] =1.0 - (float)y / winHeight * 2;
}

// Main drawing routine. Called repeatedly by GLUT's main loop
void display( void )
{
   //Clear the screen and set our initial view matrix
   glClear(GL_COLOR_BUFFER_BIT);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

   //TODO: Perform drawing here
	glColor3f(0.0, 0.0, 1.0);
	// draw a rectangle
	glBegin(GL_POLYGON);
		glVertex2f( center[0] + ratio, center[1] + ratio );
		glVertex2f( center[0] - ratio, center[1] + ratio );
		glVertex2f( center[0] - ratio, center[1] - ratio );
		glVertex2f( center[0] + ratio, center[1] - ratio );
	glEnd();

   glutSwapBuffers();
}

// mouse click event handler
void mouse(int button, int state, int x, int y)
{
	// set the mouse button flags
	// and store the mouse position to determine drag distance
	//cout <<button << " " << state << endl;
	if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
	{
		lmbDown = 1;
		center0[0] = center[0];
		center0[1] = center[1];
		win2world(oldPos, x, y);
	}
	else if (button == GLUT_LEFT_BUTTON && state == GLUT_UP)
	{
		lmbDown = 0;
		win2world(newPos, x, y);
		// update position
		center[0] = center0[0] + newPos[0] - oldPos[0];
		center[1] = center0[1] + newPos[1] - oldPos[1];

		// force redraw
		glutPostRedisplay();
	}

	if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
	{
		rmbDown = 1;
		ratio0 = ratio;
		win2world(oldPos, x, y);
	}
	else if (button == GLUT_RIGHT_BUTTON && state == GLUT_UP)
	{
		rmbDown = 0;
		win2world(newPos, x, y);
		// update size
		double r0 = sqrt(oldPos[0]*oldPos[0] + oldPos[1]*oldPos[1]),
			r1 = sqrt(newPos[0]*newPos[0] + newPos[1]*newPos[1]);
		ratio = ratio0 * r1 / r0;

		// force redraw
		glutPostRedisplay();
	}
}

// mouse move event handler
void motion(int x, int y)
{
	// LMB down + move = LMB drag
	// RMB down + move = RMB drag
	// redraw the rectangle for preview
	if (lmbDown == 1)
	{
		float tempPos[2];
		win2world(tempPos, x, y);
		// update position
		center[0] = center0[0] + tempPos[0] - oldPos[0];
		center[1] = center0[1] + tempPos[1] - oldPos[1];

		// force redraw
		glutPostRedisplay();
	}
	else if (rmbDown == 1)
	{
		float tempPos[2];
		win2world(tempPos, x, y);
		// update size
		double r0 = sqrt(oldPos[0]*oldPos[0] + oldPos[1]*oldPos[1]),
			r1 = sqrt(tempPos[0]*tempPos[0] + tempPos[1]*tempPos[1]);
		ratio = ratio0 * r1 / r0;

		// force redraw
		glutPostRedisplay();
	}
}

// Entry point - GLUT setup and initialization
int main( int argc, char** argv )
{

  glutInit( &argc, argv );
  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);
  glutInitWindowSize (winWidth, winHeight);
  glutInitWindowPosition (100, 100);
  glutCreateWindow( "Example 3" );
  glutDisplayFunc( display );

  // register a keyboard function
  glutMouseFunc( mouse );
  glutMotionFunc( motion );

  init();

  glutMainLoop();

  return 0;
}